class GitCreator < SCMCreator

  class << self

    def enabled?
      if options
        if options['path']
          if !options['git'] || File.executable?(options['git'])
            return true
          else
            Rails.logger.warn "'#{options['git']}' cannot be found/executed - ignoring '#{scm_id}"
          end
        else
          Rails.logger.warn "missing path for '#{scm_id}'"
        end
      end

      false
    end

    def repository_name_from_root_url(path)
      base_path = "#{options['path']}/"
      if path.include?("#{options['path']}")
        path.gsub(base_path, "")
      else
        nil
      end
    end

    def external_url(repository)
      name = repository_name_from_root_url(repository.root_url)
      if options['url']
        "#{options['url']}/#{name}"
      else
        nil
      end
    end

    # => git clone git@enziin.com:username/example.git
    def ssh_external_url(repository)
      name = repository_name_from_root_url(repository.root_url)
      if name
        host_name_only = Setting.host_name
        host_name_only = host_name_only.gsub("https://www.", "")
        "git@#{host_name_only}:#{name}"
      else
        nil
      end
    end

    def default_path(identifier)
      if options['git_ext']
        path(identifier) + '.git'
      else
        path(identifier)
      end
    end

    def existing_path(identifier, repository = nil)
      path = path(identifier)
      if File.directory?("#{path}.git")
        path + '.git'
      elsif File.directory?(path)
        path
      else
        false
      end
    end

    def repository_name(path)
      if Redmine::Platform.mswin?
        path_ext = "#{options['url']}\\#{User.current.login}"
      else
        path_ext = "#{options['url']}/#{User.current.login}"
      end
      base = Redmine::Platform.mswin? ? path_ext.gsub(%r{\\}, "/") : path_ext
      matches = Regexp.new("\\A#{Regexp.escape(base)}/([^/]+?)(\\.git)?/?\\z").match(path)
      matches ? matches[1] : nil
    end

    def repository_name_only_from_root_url(path)
      base_path = "#{options['path']}/"
      if path.include?("#{options['path']}")
        path = path.gsub(base_path, "")
        array_path = path.split("/")
        array_path[1]
      else
        nil
      end
    end

    def repository_exists?(identifier)
      path = path(identifier)
      File.directory?(path) || File.directory?("#{path}.git")
    end

    def new_project_repository_exists?(identifier)
      path = path(identifier)
      File.directory?(path) || File.directory?("#{path}.git")
    end

    def create_repository(path, project_id)
      args = [git_command, 'init']
      append_options(args)
      append_template(args)
      args << path
      if system(*args)
        if options['update_server_info']
          Dir.chdir(path) do
            system(git_command, 'update-server-info')
          end
        end
        hostname = Setting.host_name
        hostname_http = hostname.gsub("https", "http")
        if Rails.env.production? == false
          system("echo '#!/bin/sh' > #{path}/hooks/post-receive")
          system("echo 'curl \"#{hostname_http}:#{PORT_AUTHENTICATE_APP}/sys/hook_changesets?id=#{project_id}\"' >> #{path}/hooks/post-receive")
          system("chown -R git:git #{path}")
          system("chmod -R 777 #{path}")
          system("chmod +x #{path}/hooks/post-receive")
        else
          system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' echo '#!/bin/sh' > #{path}/hooks/post-receive")
          system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' echo 'curl \"#{hostname_http}:#{PORT_AUTHENTICATE_APP}/sys/hook_changesets?id=#{project_id}\"' >> #{path}/hooks/post-receive")
          system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chown -R git:git #{path}")
          system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chmod -R 777 #{path}")
          system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chmod +x #{path}/hooks/post-receive")
        end
        true
      else
        false
      end
    end

    private

    def git_command
      options['git']
    end
  end
end
