class SubversionCreator < SCMCreator

  class << self

    def scm_id
      'svn'
    end

    def enabled?
      if options
        if options['path']
          if !options['svnadmin'] || File.executable?(options['svnadmin'])
            return true
          else
            Rails.logger.warn "'#{options['svnadmin']}' cannot be found/executed - ignoring '#{scm_id}"
          end
        else
          Rails.logger.warn "missing path for '#{scm_id}'"
        end
      end

      false
    end

    def access_url(path, repository = nil)
      if options['append']
        access_root_url(path, repository) + '/' + options['append']
      else
        access_root_url(path, repository)
      end
    end

    def access_root_url(path, repository = nil)
      'file://' + (Redmine::Platform.mswin? ? '/' + path.gsub(%r{\\}, "/") : path)
    end

    def external_url(repository)
      name = repository_name_from_root_url(repository.root_url)
      if options['url']
        "#{options['url']}/#{name}"
      else
        nil
      end
    end

    def new_external_url(project_identifier)
      if options['url']
        "#{options['url']}/#{project_identifier}"
      else
        nil
      end
    end

    # checks if repository already exists (was created for Git which can add .git extension)
    def repository_exists?(path)
      File.directory?(path)
    end

    # checks if repository already exists (was created for Git which can add .git extension)
    def new_project_repository_exists?(project_id)
      path_check = "#{options['path']}/#{project_id}"
      File.directory?(path_check)
    end

    def repository_name(path)
      base_path = "#{options['url']}/"
      if path.include?("#{options['url']}")
        path.gsub(base_path, "")
      else
        nil
      end
    end

    def repository_name_from_root_url(path)
      base_path = "file://#{options['path']}/"
      if path.include?("#{options['path']}")
        path.gsub(base_path, "")
      else
        nil
      end
    end

    def repository_name_only_from_root_url(path)
      base_path = "file://#{options['path']}/"
      if path.include?("#{options['path']}")
        path = path.gsub(base_path, "")
        path
      else
        nil
      end
    end

    # returns local path
    def path(identifier)
      if Redmine::Platform.mswin?
        # Assuming path is in Windows style (contains \'s)
        path_dir = "#{options['path']}\\"
        path_ext = "#{options['path']}\\#{identifier}"
      else
        path_dir = "#{options['path']}"
        path_ext = "#{options['path']}/#{identifier}"
      end
      if !File.directory?(path_dir)
        Dir.mkdir(path_dir.downcase, 0775)
      end
      path_ext.downcase
    end

    # get path of existing repository
    def existing_path(identifier, repository = nil)
      if File.directory?(default_path(identifier))
        default_path(identifier)
      else
        nil
      end
    end

    # constructs default path using project identifier
    def default_path(identifier)
      path(identifier)
    end

    def repository_format
      if Redmine::Platform.mswin?
        path_ext = "#{options['path']}"
      else
        path_ext = "#{options['path']}"
      end
      path = Redmine::Platform.mswin? ? '/' + path_ext.gsub(%r{\\}, "/") : path_ext
      "file://#{path}/<#{l(:label_repository_format)}>/"
    end

    def create_repository(path, project_id)
      args = [svnadmin_command, 'create', path]
      append_options(args)

      system(*args)

      project = Project.find(project_id)
      if project.present?
        project_type = project.project_type
      else
        project_type = 'IT'
      end
      if project_type == 'IT'
        system("svn mkdir --parents -m \"Initialize the repository\" file://#{path}/trunk file://#{path}/branches file://#{path}/tags file://#{path}/sandbox")
      else
        system("svn mkdir --parents -m \"Initialize the repository\" file://#{path}/Documents file://#{path}/MCAD file://#{path}/ECAD file://#{path}/Firmware file://#{path}/Software")
      end

      hostname = Setting.host_name
      hostname_http = hostname.gsub("https", "http")

      if Rails.env.production? == false
        system("echo '#!/bin/sh' > #{path}/hooks/post-commit")
        system("echo 'curl \"#{hostname_http}:#{PORT_AUTHENTICATE_APP}/sys/hook_changesets?id=#{project_id}\"' >> #{path}/hooks/post-commit")
        system("chown -R svn:svn #{path}")
        system("chmod -R 777 #{path}")
        system("chmod +x #{path}/hooks/post-commit")
      else
        system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' echo '#!/bin/sh' > #{path}/hooks/post-commit")
        system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' echo 'curl \"#{hostname_http}:#{PORT_AUTHENTICATE_APP}/sys/hook_changesets?id=#{project_id}\"' >> #{path}/hooks/post-commit")
        system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chown -R svn:svn #{path}/hooks/post-commit")
        system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chmod -R 777 #{path}/hooks/post-commit")
        system("sshpass -p '#{PASSWORD_DEVELOPER_USER}' chmod +x #{path}/hooks/post-commit")
      end
    end

    private

    def svnadmin_command
      options['svnadmin'] || 'svnadmin'
    end
  end
end
