require_dependency 'repositories_helper'

module ScmRepositoriesHelperPatch

  def self.included(base)
    base.send(:include, InstanceMethods)
    base.class_eval do
      unloadable

      alias_method :repository_field_tags_without_add, :repository_field_tags
      alias_method :repository_field_tags, :repository_field_tags_with_add

      alias_method :subversion_field_tags_without_add, :subversion_field_tags
      alias_method :subversion_field_tags, :subversion_field_tags_with_add

      alias_method :mercurial_field_tags_without_add, :mercurial_field_tags
      alias_method :mercurial_field_tags, :mercurial_field_tags_with_add

      alias_method :git_field_tags_without_add, :git_field_tags
      alias_method :git_field_tags, :git_field_tags_with_add

      alias_method :bazaar_field_tags_without_add, :bazaar_field_tags
      alias_method :bazaar_field_tags, :bazaar_field_tags_with_add

      if method_defined?(:scm_path_info_tag)
        alias_method :scm_path_info_tag_without_external, :scm_path_info_tag
        alias_method :scm_path_info_tag, :scm_path_info_tag_with_external
      end

    end
  end

  module InstanceMethods

    def repository_field_tags_with_add(form, repository)
      reptags = repository_field_tags_without_add(form, repository)

      button_disabled = repository.class.respond_to?(:scm_available) ? !repository.class.scm_available : false

      if ScmConfig['only_creator']
        interface = SCMCreator.interface(repository)

        if interface && (interface < SCMCreator) && interface.enabled? && repository.new_record?
          button_disabled = true
        end
      end

      if request.xhr?
        reptags << javascript_tag("$('input[type=submit][name=commit]')." + (button_disabled ? "attr('disabled','disabled')" : "removeAttr('enable')") + ";")
      else
        reptags << javascript_tag("$(document).ready(function() { $('input[type=submit][name=commit]')." + (button_disabled ? "attr('disabled','disabled')" : "removeAttr('enable')") + "; });")
      end

      reptags.html_safe
    end

    def subversion_field_tags_with_add(form, repository)
      svntags = ''

      if repository.new_record? && SubversionCreator.enabled? && !limit_exceeded
        svntags << hidden_field_tag(:operation, '', :id => 'repository_operation')
      elsif !respond_to?(:scm_path_info_tag) && !repository.new_record? && repository.created_with_scm && SubversionCreator.enabled?
        url = SubversionCreator.external_url(repository)
        if url
          svntags.sub!('(file:///, http://, https://, svn://, svn+[tunnelscheme]://)', url)
        end
      end
      svntags << submit_tag("Submit", :onclick => "$('#repository_operation').val('add');", :id => :scm_creator_button)

      svntags
    end

    def mercurial_field_tags_with_add(form, repository)
      hgtags = mercurial_field_tags_without_add(form, repository)

      if repository.new_record? && MercurialCreator.enabled? && !limit_exceeded
        hgtags << hidden_field_tag(:operation, '', :id => 'repository_operation')
        unless request.post?
          path = MercurialCreator.access_root_url(MercurialCreator.default_path(@project.identifier), repository)
          if MercurialCreator.repository_exists?(@project.identifier)
            path << '.' + @project.repositories.select {|r| r.created_with_scm}.size.to_s
          end
          hgtags << javascript_tag("$('#repository_url').val('#{escape_javascript(path)}');")
        end
        hgtags << javascript_tag("$('#repository_url')")

      elsif !respond_to?(:scm_path_info_tag) && !repository.new_record? && repository.created_with_scm && MercurialCreator.enabled?
        url = MercurialCreator.external_url(repository)
        if url
          if hgtags.include?(l(:text_mercurial_repository_note))
            hgtags.sub!(l(:text_mercurial_repository_note), url)
          else
            hgtags.sub!('</p>', '<br />' + url + '</p>')
          end
        end
      end
      hgtags << submit_tag("Submit", :onclick => "$('#repository_operation').val('add');", :id => :scm_creator_button)

      hgtags
    end

    def bazaar_field_tags_with_add(form, repository)
      bzrtags = bazaar_field_tags_without_add(form, repository)

      if repository.new_record? && BazaarCreator.enabled? && !limit_exceeded
        bzrtags << hidden_field_tag(:operation, '', :id => 'repository_operation')
        unless request.post?
          path = BazaarCreator.access_root_url(BazaarCreator.default_path(@project.identifier), repository)
          if BazaarCreator.repository_exists?(@project.identifier)
            path << '.' + @project.repositories.select {|r| r.created_with_scm}.size.to_s
          end
          bzrtags << javascript_tag("$('#repository_url').val('#{escape_javascript(path)}');")
          if BazaarCreator.options['log_encoding']
            bzrtags << javascript_tag("$('#repository_log_encoding').val('#{escape_javascript(BazaarCreator.options['log_encoding'])}');")
          end
        end
        bzrtags << javascript_tag("$('#repository_url')")

      elsif !respond_to?(:scm_path_info_tag) && !repository.new_record? && repository.created_with_scm && BazaarCreator.enabled?
        url = BazaarCreator.external_url(repository)
        if url
          bzrtags.sub!('</p>', '<br />' + url + '</p>')
        end
      end
      bzrtags << submit_tag("Submit", :onclick => "$('#repository_operation').val('add');", :id => :scm_creator_button)

      bzrtags
    end

    def git_field_tags_with_add(form, repository)
      gittags = ''

      if repository.new_record? && GitCreator.enabled? && !limit_exceeded
        gittags << hidden_field_tag(:operation, '', :id => 'repository_operation')
      elsif !respond_to?(:scm_path_info_tag) && !repository.new_record? && repository.created_with_scm && GitCreator.enabled?
        url = GitCreator.external_url(repository)
        if url
          if gittags.include?(l(:text_git_repository_note))
            gittags.sub!(l(:text_git_repository_note), url)
          else
            gittags.sub!('</p>', '<br />' + url + '</p>')
          end
        end
      end
      gittags << submit_tag("Submit", :onclick => "$('#repository_operation').val('add');", :id => :scm_creator_button)

      gittags
    end

    def github_field_tags(form, repository)
      urltag = form.text_field(:url, :size => 60, placeholder: "Copy path of repository from Github.",
                               :required => true,
                               :disabled => !repository.safe_attribute?('url'))

      if repository.new_record? && GithubCreator.enabled? && !limit_exceeded
        urltag << ' '.html_safe
        urltag << hidden_field_tag(:operation, '', :id => 'repository_operation')

        note = content_tag('em', "https://github.com/user_name/example.git", :class => 'info')
      elsif repository.new_record?
        note = content_tag('em', "https://github.com/user_name/example.git", :class => 'info')
      end

      githubtags = content_tag('p', urltag + note)
      if !Setting.autofetch_changesets? && GithubCreator.can_register_hook?
        githubtags << content_tag('p', form.check_box(:extra_register_hook, :disabled => repository.extra_hook_registered) + ' ' +
            l(:text_github_register_hook_note))
      end
      githubtags << submit_tag(l(:label_repository_import))
      githubtags
    end

    def scm_path_info_tag_with_external(repository)
      if !repository.new_record? && repository.created_with_scm
        interface = SCMCreator.interface(repository)
        if interface && (url = interface.external_url(repository))
          return content_tag('em', url, :class => 'info')
        end
      end
      scm_path_info_tag_without_external(repository)
    end

    private

    def limit_exceeded
      ScmConfig['max_repos'] && ScmConfig['max_repos'].to_i > 0 &&
          @project.repositories.select {|r| r.created_with_scm}.size >= ScmConfig['max_repos'].to_i
    end
  end
end
